@@ -38,7 +38,7 @@ static void set_buddies_offline(PurpleBuddy *buddy,
 	}
 }
 
-static void buddy_icon_cb(FacebookAccount *fba, gchar *data, gsize data_len,
+static void buddy_icon_cb(FacebookAccount *fba, const gchar *data, gsize data_len,
 		gpointer user_data)
 {
 	gchar *buddyname;
@@ -110,23 +110,18 @@ static GList *get_buddies(FacebookAccount *fba, const gchar *uid,
 	return buddies;
 }
 
-static void process_buddy_icon(FacebookAccount *fba, PurpleBuddy *buddy,
-	JsonObject *userInfo)
+static void process_buddy_icon(FacebookAccount *fba, FacebookBuddy *fbuddy,
+	const gchar *buddy_icon_url)
 {
-	FacebookBuddy *fbuddy;
-	gchar *buddy_icon_url;
+	PurpleBuddy *buddy;
 	gchar *icon_host;
 	gchar *icon_path, *real_path;
 	gchar *search_tmp;
 
-	fbuddy = buddy->proto_data;
+	buddy = fbuddy->buddy;
 	
-	/* Set the buddy icon (if it hasn't changed) */
-	buddy_icon_url = g_strdup(json_node_get_string(json_object_get_member(
-			userInfo, "thumbSrc")));
 	/* Seperate the URL into pieces */
 	purple_url_parse(buddy_icon_url, &icon_host, NULL, &icon_path, NULL, NULL);
-	g_free(buddy_icon_url);
 	
 	if (icon_path != NULL && icon_path[0] != '/')
 	{
@@ -155,7 +150,13 @@ static void process_buddy_icon(FacebookAccount *fba, PurpleBuddy *buddy,
 			/* bigger icon at /profile6/1845/74/n800753867_2878.jpg */
 			search_tmp = strstr(icon_path, "/q");
 			if (search_tmp)
-				*(search_tmp + 1) = 'n';			
+				*(search_tmp + 1) = 'n';
+			else
+			{
+				search_tmp = strstr(icon_path, "_q.jpg");
+				if (search_tmp)
+					*(search_tmp + 1) = 'n';
+			}
 			purple_debug_info("facebook", "buddy %s has a new buddy icon at http://%s%s\n", buddy->name, icon_host, icon_path);
 			/* Fetch their icon */
 			fb_post_or_get(fba, FB_METHOD_GET, icon_host,
@@ -215,10 +216,13 @@ static void process_buddies(FacebookAccount *fba, GHashTable *online_buddies_lis
 	for (cur = buddies; cur != NULL; cur = cur->next)
 	{
 		PurpleBuddy *buddy;
+		FacebookBuddy *fbuddy;
 
 		buddy = (PurpleBuddy *)cur->data;
+		fbuddy = buddy->proto_data;
 
-		process_buddy_icon(fba, buddy, userInfo);
+		process_buddy_icon(fba, fbuddy, json_node_get_string(
+			json_object_get_member(userInfo, "thumbSrc")));
 
 		purple_presence_set_idle(purple_buddy_get_presence(buddy),
 				idle, 0);
@@ -272,7 +276,7 @@ static void process_notifications(FacebookAccount *fba,
 	}
 }
 
-static void got_status_stream_cb(FacebookAccount *fba, gchar *data,
+static void got_status_stream_cb(FacebookAccount *fba, const gchar *data,
 		gsize data_len, gpointer userdata)
 {
 	gchar *error = NULL;
@@ -317,7 +321,7 @@ static void got_status_stream_cb(FacebookAccount *fba, gchar *data,
 			objnode, "html"));
 	//purple_debug_misc("facebook", "html data\n%s\n", html);
 	
-	messages = g_strsplit(html, "/h3>", -1);
+	messages = g_strsplit(html, "/h6>", -1);
 	for(i = 0; messages[i]; i++)
 	{
 		message = messages[i];
@@ -389,7 +393,7 @@ static void got_status_stream_cb(FacebookAccount *fba, gchar *data,
 	json_parser_free(parser);
 }
 
-static void got_buddy_list_cb(FacebookAccount *fba, gchar *data,
+static void got_buddy_list_cb(FacebookAccount *fba, const gchar *data,
 		gsize data_len, gpointer userdata)
 {
 	GSList *buddies_list;
@@ -531,6 +535,85 @@ gboolean fb_get_buddy_list(gpointer data)
 	return TRUE;
 }
 
+static void got_full_buddy_list(FacebookAccount *fba, const gchar *data,
+		gsize data_len, gpointer userdata)
+{
+	int i;
+	PurpleGroup *fb_group;
+	JsonParser *parser;
+	gchar *error = NULL;
+	JsonObject *objnode, *node;
+	JsonArray *entries;
+	
+	purple_debug_info("facebook", "parsing full buddy list\n");
+
+	if (fba == NULL)
+		return;
+
+	parser = fb_get_parser(data, data_len);
+	if (parser == NULL)
+		return;
+
+	purple_debug_misc("facebook", "full buddy list\n%s\n", data);
+
+	objnode = fb_get_json_object(parser, &error);
+	if (!json_object_has_member(objnode, "payload"))
+	{
+		json_parser_free(parser);
+		return;
+	}
+	objnode = json_node_get_object(json_object_get_member(
+			objnode, "payload"));
+	if (!json_object_has_member(objnode, "entries"))
+	{
+		json_parser_free(parser);
+		return;
+	}
+	entries = json_node_get_array(json_object_get_member(
+			objnode, "entries"));
+			
+	fb_group = purple_find_group(DEFAULT_GROUP_NAME);
+	if (fb_group == NULL)
+	{
+		fb_group = purple_group_new(DEFAULT_GROUP_NAME);
+		purple_blist_add_group(fb_group, NULL);
+	}
+	for(i = 0; i < json_array_get_length(entries); i++)
+	{
+		node = json_node_get_object(json_array_get_element(entries, i));
+		const gchar *type = json_node_get_string(json_object_get_member(node, "ty"));
+		if (type[0] != 'u')
+			continue;
+		const gchar *uid = json_node_get_string(json_object_get_member(node, "i"));
+		if (purple_find_buddy(fba->account, uid))
+			continue;
+		
+		const gchar *name = json_node_get_string(json_object_get_member(node, "t"));
+		PurpleBuddy *buddy = purple_buddy_new(fba->account, uid, name);
+		purple_blist_add_buddy(buddy, NULL, fb_group, NULL);
+		FacebookBuddy *fbuddy = g_new0(FacebookBuddy, 1);
+		fbuddy->buddy = buddy;
+		fbuddy->fba = fba;
+		fbuddy->uid = atoll(uid);
+		fbuddy->name = g_strdup(name);
+		buddy->proto_data = fbuddy;
+		
+		const gchar *thumb_url = json_node_get_string(json_object_get_member(node, "it"));
+		process_buddy_icon(fba, fbuddy, thumb_url);
+	}
+
+	json_parser_free(parser);	
+}
+
+void fb_get_full_buddy_list(FacebookAccount *fba)
+{
+	gchar *url;
+	
+	url = g_strdup_printf("/ajax/typeahead_search.php?u=%" G_GINT64_FORMAT "&__a=1", fba->uid);
+	fb_post_or_get(fba, FB_METHOD_GET, NULL, url, NULL, got_full_buddy_list, NULL, FALSE);
+	g_free(url);
+}
+
 void fb_blist_poke_buddy(PurpleBlistNode *node, gpointer data)
 {
 	PurpleBuddy *buddy;
@@ -594,6 +677,7 @@ void fb_blist_init(FacebookAccount *fba)
 	fba->buddy_list_timer = purple_timeout_add_seconds(60,
 			fb_get_buddy_list, fba);
 
+	fb_get_full_buddy_list(fba);
 }
 
 void fb_blist_destroy(FacebookAccount *fba)
