@@ -21,6 +21,7 @@
 #include "fb_connection.h"
 
 static void fb_attempt_connection(FacebookConnection *);
+static void fb_next_connection(FacebookAccount *fba);
 
 #ifdef HAVE_ZLIB
 #include <zlib.h>
@@ -98,6 +99,8 @@ static gchar *fb_gunzip(const guchar *gzip_data, ssize_t *len_ptr)
 
 	return g_string_free(output_string, FALSE);
 }
+#else /* !HAVE_ZLIB */
+#warning You really want to compile with -DHAVE_ZLIB
 #endif
 
 void fb_connection_destroy(FacebookConnection *fbconn)
@@ -227,10 +230,12 @@ static void fb_post_or_get_readdata_cb(gpointer data, gint source,
 		PurpleInputCondition cond)
 {
 	FacebookConnection *fbconn;
+	FacebookAccount *fba;
 	gchar buf[4096];
 	ssize_t len;
 
 	fbconn = data;
+	fba = fbconn->fba;
 
 	if (fbconn->method & FB_METHOD_SSL) {
 		len = purple_ssl_read(fbconn->ssl_conn,
@@ -290,6 +295,8 @@ static void fb_post_or_get_readdata_cb(gpointer data, gint source,
 	fb_connection_process_data(fbconn);
 
 	fb_connection_destroy(fbconn);
+	
+	fb_next_connection(fba);
 }
 
 static void fb_post_or_get_ssl_readdata_cb (gpointer data,
@@ -420,7 +427,7 @@ static void fb_cookie_foreach_cb(gchar *cookie_name,
 /**
  * Serialize the fba->cookie_table hash table to a string.
  */
-static gchar *fb_cookies_to_string(FacebookAccount *fba)
+gchar *fb_cookies_to_string(FacebookAccount *fba)
 {
 	GString *str;
 
@@ -593,9 +600,25 @@ void fb_post_or_get(FacebookAccount *fba, FacebookMethod method,
 	fbconn->fd = -1;
 	fbconn->connection_keepalive = keepalive;
 	fbconn->request_time = time(NULL);
-	fba->conns = g_slist_prepend(fba->conns, fbconn);
+	
+	g_queue_push_head(fba->waiting_conns, fbconn);
+	fb_next_connection(fba);
+}
 
-	fb_attempt_connection(fbconn);
+static void fb_next_connection(FacebookAccount *fba)
+{
+	FacebookConnection *fbconn;
+	
+	g_return_if_fail(fba != NULL);	
+	
+	if (!g_queue_is_empty(fba->waiting_conns))
+	{
+		if(g_slist_length(fba->conns) < FB_MAX_CONNECTIONS)
+		{
+			fbconn = g_queue_pop_tail(fba->waiting_conns);
+			fb_attempt_connection(fbconn);
+		}
+	}
 }
 
 static void fb_attempt_connection(FacebookConnection *fbconn)
@@ -621,6 +644,8 @@ static void fb_attempt_connection(FacebookConnection *fbconn)
 	 * runs at blinding speed.  Slow it down. */
 	/* TODO/FIXME: this doesn't retry properly on non-ssl connections */
 #endif
+	
+	fba->conns = g_slist_prepend(fba->conns, fbconn);
 
 	if (fbconn->method & FB_METHOD_SSL) {
 		fbconn->ssl_conn = purple_ssl_connect(fba->account, fbconn->hostname,
