@@ -91,7 +91,7 @@ void fb_conversation_handle_message(FacebookAccount *fba, const char *from,
  * HISTORY CODE                                                              *
  *****************************************************************************/
 
-static void fb_history_fetch_cb(FacebookAccount *fba, gchar *data,
+static void fb_history_fetch_cb(FacebookAccount *fba, const gchar *data,
 	gsize data_len, gpointer userdata)
 {
 	JsonParser *parser;
