@@ -77,7 +77,7 @@ static void fb_auth_reject_cb(gpointer data)
 	g_free(buddy_uid);
 }
 
-static void fb_check_friend_request_cb(FacebookAccount *fba, gchar *data,
+static void fb_check_friend_request_cb(FacebookAccount *fba, const gchar *data,
 		gsize data_len, gpointer user_data)
 {
 	const char *uid_pre_text = "class=\"confirm\" id=\"friend_connect_";
@@ -89,7 +89,7 @@ static void fb_check_friend_request_cb(FacebookAccount *fba, gchar *data,
 	gchar *msg;
 	gchar *msg_plain;
 	FacebookBuddy *buddy;
-	gchar *search_start = data;
+	const gchar *search_start = data;
 
 	g_return_if_fail(data_len > 0);
 	g_return_if_fail(data != NULL);
