@@ -40,7 +40,6 @@ struct _FacebookOutgoingMessage {
 
 static gboolean fb_send_im_fom(FacebookOutgoingMessage *msg);
 static gboolean fb_resend_im_fom(FacebookOutgoingMessage *msg);
-static gboolean fb_get_new_messages(FacebookAccount *fba);
 
 static FacebookOutgoingMessage *fb_msg_create(FacebookAccount *fba)
 {
@@ -150,7 +149,7 @@ static void parse_new_messages(PurpleConnection *pc, FacebookAccount *fba, JsonA
 	}
 }
 								  
-static void got_new_messages(FacebookAccount *fba, gchar *data,
+static void got_new_messages(FacebookAccount *fba, const gchar *data,
 		gsize data_len, gpointer userdata)
 {
 	JsonParser *parser;
@@ -258,7 +257,7 @@ static void got_new_messages(FacebookAccount *fba, gchar *data,
  * called ONCE and only ONCE.  After that the timers will take care of
  * themselves until final cleanup.
  */
-static gboolean fb_get_new_messages(FacebookAccount *fba)
+gboolean fb_get_new_messages(FacebookAccount *fba)
 {
 	time_t now;
 	gchar *fetch_url;
@@ -304,7 +303,7 @@ static gboolean fb_get_new_messages(FacebookAccount *fba)
 	return FALSE;
 }
 
-static void fb_send_im_cb(FacebookAccount *fba, gchar *data, gsize data_len, gpointer user_data)
+static void fb_send_im_cb(FacebookAccount *fba, const gchar *data, gsize data_len, gpointer user_data)
 {
 	FacebookOutgoingMessage *msg = user_data;
 	JsonParser *parser;
@@ -405,7 +404,7 @@ int fb_send_im(PurpleConnection *pc, const gchar *who, const gchar *message, Pur
 	return 1;
 }
 
-void got_reconnect_json(FacebookAccount *fba, gchar *data, gsize data_len, gpointer userdata)
+void got_reconnect_json(FacebookAccount *fba, const gchar *data, gsize data_len, gpointer userdata)
 {
 	JsonParser *parser;
 	JsonObject *objnode;
@@ -462,7 +461,7 @@ gboolean fb_reconnect(FacebookAccount *fba)
 	return FALSE;
 }
 
-static void got_form_id_page(FacebookAccount *fba, gchar *data, gsize data_len, gpointer userdata)
+static void got_form_id_page(FacebookAccount *fba, const gchar *data, gsize data_len, gpointer userdata)
 {
 	const gchar *start_text = "id=\"post_form_id\" name=\"post_form_id\" value=\"";
 	const gchar *dtsg_start = "fb_dtsg:\"";
@@ -471,6 +470,7 @@ static void got_form_id_page(FacebookAccount *fba, gchar *data, gsize data_len,
 	gchar *post_form_id;
 	gchar *channel = NULL;
 	gchar *tmp = NULL;
+	FacebookFunc callback;
 	
 	/* NULL data crashes on Windows */
 	if (data == NULL)
@@ -516,22 +516,16 @@ static void got_form_id_page(FacebookAccount *fba, gchar *data, gsize data_len,
 		g_free(fba->channel_number);
 		fba->channel_number = channel;
 	}
-
-	tmp = g_strdup_printf("visibility=true&post_form_id=%s", post_form_id);
-	fb_post_or_get(fba, FB_METHOD_POST, "apps.facebook.com", "/ajax/chat/settings.php", tmp, NULL, NULL, FALSE);
-	g_free(tmp);
 	
-	if (channel == NULL)
+	if (userdata)
 	{
-		/* Grab new channel number */
-		fb_reconnect(fba);
-	} else {
-		fb_get_new_messages(fba);
+		callback = userdata;
+		callback(fba);
 	}
 }
 
-gboolean fb_get_post_form_id(FacebookAccount *fba)
+gboolean fb_get_post_form_id(FacebookAccount *fba, FacebookFunc callback)
 {
-	fb_post_or_get(fba, FB_METHOD_GET, NULL, "/presence/popout.php", NULL, got_form_id_page, NULL, FALSE);
+	fb_post_or_get(fba, FB_METHOD_GET, NULL, "/presence/popout.php", NULL, got_form_id_page, callback, FALSE);
 	return FALSE;
 }
