@@ -29,7 +29,7 @@
 #include "fb_search.h"
 #include "fb_friendlist.h"
 
-static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
+static void fb_login_cb(FacebookAccount *fba, const gchar *response, gsize len,
 		gpointer userdata);
 static void fb_close(PurpleConnection *pc);
 static void fb_buddy_free(PurpleBuddy *buddy);
@@ -96,6 +96,23 @@ static GList *fb_statuses(PurpleAccount *account)
 	return types;
 }
 
+void fb_post_form_id_cb(FacebookAccount *fba)
+{
+	gchar *tmp;
+	
+	tmp = g_strdup_printf("visibility=true&post_form_id=%s", fba->post_form_id);
+	fb_post_or_get(fba, FB_METHOD_POST, "apps.facebook.com", "/ajax/chat/settings.php", tmp, NULL, NULL, FALSE);
+	g_free(tmp);
+	
+	if (fba->channel_number == NULL)
+	{
+		/* Grab new channel number */
+		fb_reconnect(fba);
+	} else {
+		fb_get_new_messages(fba);
+	}
+}
+
 static gboolean fb_get_messages_failsafe(FacebookAccount *fba)
 {
 	if (fba->last_messages_download_time < (time(NULL) - (60*5))) {
@@ -103,7 +120,7 @@ static gboolean fb_get_messages_failsafe(FacebookAccount *fba)
 		 * something is probably wrong */
 		purple_debug_warning("facebook",
 				"executing message check failsafe\n");
-		fb_get_post_form_id(fba);
+		fb_get_post_form_id(fba, fb_post_form_id_cb);
 	}
 
 	return TRUE;
@@ -173,7 +190,7 @@ void fb_login_captcha_ok_cb(PurpleConnection *pc, PurpleRequestFields *fields)
 	fba->captcha_session = NULL;
 }
 
-static void fb_login_captcha_image_cb(FacebookAccount *fba, gchar *response, 
+static void fb_login_captcha_image_cb(FacebookAccount *fba, const gchar *response, 
 		gsize len, gpointer userdata)
 {
 	PurpleRequestFields *fields;
@@ -200,7 +217,7 @@ static void fb_login_captcha_image_cb(FacebookAccount *fba, gchar *response,
 	);
 }
 
-static void fb_login_captcha_cb(FacebookAccount *fba, gchar *response, 
+static void fb_login_captcha_cb(FacebookAccount *fba, const gchar *response, 
 		gsize len, gpointer userdata)
 {
 	const gchar *challenge_start = "challenge : '";
@@ -220,7 +237,7 @@ static void fb_login_captcha_cb(FacebookAccount *fba, gchar *response,
 	}
 }
 
-static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
+static void fb_login_cb(FacebookAccount *fba, const gchar *response, gsize len,
 		gpointer userdata)
 {
 	gchar *user_cookie;
@@ -284,6 +301,16 @@ static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
 		return;
 	}
 
+	
+	/* Check for new 'device login' security setting */
+	if (g_hash_table_lookup(fba->cookie_table, "lgnntfy") && !g_hash_table_lookup(fba->cookie_table, "c_user"))
+	{
+		/* Facebook may need us to identify ourselves */
+		fb_post_or_get(fba, FB_METHOD_POST | FB_METHOD_SSL, "login.facebook.com",
+			"/loginnotify/setup_machine.php", "machinename=libpurple&remembercomputer=1", fb_login_cb, NULL, FALSE);
+		return;
+	}	
+
 	purple_connection_update_progress(fba->pc, _("Authenticating"), 2, 3);
 
 	/* Look for our uid */
@@ -305,7 +332,7 @@ static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
 	purple_connection_set_state(fba->pc, PURPLE_CONNECTED);
 
 	/* This will kick off our long-poll message retrieval loop */
-	fb_get_post_form_id(fba);
+	fb_get_post_form_id(fba, &fb_post_form_id_cb);
 	fb_check_friend_requests(fba);
 
 	/* periodically check for people adding you to their facebook friend list */
@@ -337,13 +364,62 @@ static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
 	fb_conversation_init(fba);
 }
 
-static void fb_login(PurpleAccount *account)
+gboolean fb_do_http_login(FacebookAccount *fba, 
+		FacebookProxyCallbackFunc callback_func, gpointer user_data)
 {
-	FacebookAccount *fba;
 	gchar *postdata, *encoded_username, *encoded_password, *encoded_charset_test;
 	const gchar* const *languages;
 	const gchar *locale;
 	
+	if (fba == NULL)
+		return FALSE;
+	
+	/* Error localized in libpurple jabber.c */
+	if (!purple_ssl_is_supported()) {
+		purple_connection_error_reason (fba->pc,
+										PURPLE_CONNECTION_ERROR_NO_SSL_SUPPORT,
+										_("Server requires TLS/SSL for login.  No TLS/SSL support found."));
+		return FALSE;
+	}
+	
+	if (fba->cookie_table == NULL)
+		fba->cookie_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	if (fba->hostname_ip_cache == NULL)
+		fba->hostname_ip_cache = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	if (fba->waiting_conns == NULL)
+		fba->waiting_conns = g_queue_new();
+	
+	g_hash_table_replace(fba->cookie_table, g_strdup("test_cookie"), g_strdup("1"));
+	g_hash_table_replace(fba->cookie_table, g_strdup("lsd"), g_strdup("abcde"));
+	
+	encoded_username = g_strdup(purple_url_encode(purple_account_get_username(fba->account)));
+	encoded_password = g_strdup(purple_url_encode(purple_account_get_password(fba->account)));
+	encoded_charset_test = g_strdup(purple_url_encode("€,´,€,´,水,Д,Є"));
+	languages = g_get_language_names();
+	locale = languages[0];
+	if (locale == NULL || g_str_equal(locale, "C"))
+		locale = "en_US";
+	
+	postdata = g_strdup_printf("charset_test=%s&locale=%s&email=%s&pass=%s&"
+							   "pass_placeHolder=Password&persistent=1&"
+							   "login=Login&charset_test=%s&lsd=abcde",
+							   encoded_charset_test, locale, encoded_username, 
+							   encoded_password, encoded_charset_test);
+	g_free(encoded_username);
+	g_free(encoded_password);
+	g_free(encoded_charset_test);
+	
+	fb_post_or_get(fba, FB_METHOD_POST | FB_METHOD_SSL, "login.facebook.com",
+				   "/login.php?login_attempt=1&_fb_noscript=1", postdata, callback_func, user_data, FALSE);
+	g_free(postdata);
+	
+	return TRUE;
+}
+
+static void fb_login(PurpleAccount *account)
+{
+	FacebookAccount *fba;
+	
 	/* Create account and initialize state */
 	fba = g_new0(FacebookAccount, 1);
 	fba->account = account;
@@ -359,44 +435,12 @@ static void fb_login(PurpleAccount *account)
 	fba->auth_buddies = g_hash_table_new_full(g_str_hash, g_str_equal,
 			g_free, NULL);
 
-	g_hash_table_replace(fba->cookie_table, g_strdup("test_cookie"),
-			g_strdup("1"));
-
 	account->gc->proto_data = fba;
 
-	/* Error localized in libpurple jabber.c */
-	if (!purple_ssl_is_supported()) {
-		purple_connection_error_reason (purple_account_get_connection(account),
-				PURPLE_CONNECTION_ERROR_NO_SSL_SUPPORT,
-				_("Server requires TLS/SSL for login.  No TLS/SSL support found."));
-		return;
-	}
-
 	purple_connection_set_state(fba->pc, PURPLE_CONNECTING);
 	purple_connection_update_progress(fba->pc, _("Connecting"), 1, 3);
 
-	encoded_username = g_strdup(purple_url_encode(
-			purple_account_get_username(fba->account)));
-	encoded_password = g_strdup(purple_url_encode(
-			purple_account_get_password(fba->account)));
-	encoded_charset_test = g_strdup(purple_url_encode("€,´,€,´,水,Д,Є"));
-	languages = g_get_language_names();
-	locale = languages[0];
-	if (locale == NULL || g_str_equal(locale, "C"))
-		locale = "en_US";
-
-	g_hash_table_replace(fba->cookie_table, g_strdup("lsd"), g_strdup("abcde"));
-
-	postdata = g_strdup_printf(
-			"charset_test=%s&locale=%s&email=%s&pass=%s&pass_placeHolder=Password&persistent=1&login=Login&charset_test=%s&lsd=abcde",
-			encoded_charset_test, locale, encoded_username, encoded_password, encoded_charset_test);
-	g_free(encoded_username);
-	g_free(encoded_password);
-	g_free(encoded_charset_test);
-
-	fb_post_or_get(fba, FB_METHOD_POST | FB_METHOD_SSL, "login.facebook.com",
-			"/login.php?login_attempt=1&_fb_noscript=1", postdata, fb_login_cb, NULL, FALSE);
-	g_free(postdata);
+	fb_do_http_login(fba, fb_login_cb, NULL);
 }
 
 static void fb_close(PurpleConnection *pc)
@@ -438,7 +482,14 @@ static void fb_close(PurpleConnection *pc)
 	if (fba->perpetual_messages_timer) {
 		purple_timeout_remove(fba->perpetual_messages_timer);
 	}
-
+	
+	purple_debug_info("facebook", "destroying %d waiting connections\n",
+					  g_queue_get_length(fba->waiting_conns));
+	
+	while (!g_queue_is_empty(fba->waiting_conns))
+		fb_connection_destroy(g_queue_pop_tail(fba->waiting_conns));
+	g_queue_free(fba->waiting_conns);
+	
 	purple_debug_info("facebook", "destroying %d incomplete connections\n",
 			g_slist_length(fba->conns));
 
@@ -457,6 +508,7 @@ static void fb_close(PurpleConnection *pc)
 		fb_cancel_resending_messages(fba);
 	}
 
+	g_hash_table_destroy(fba->sent_messages_hash);
 	g_hash_table_destroy(fba->cookie_table);
 	g_hash_table_destroy(fba->hostname_ip_cache);
 	g_hash_table_destroy(fba->auth_buddies);
@@ -548,9 +600,13 @@ static void fb_set_status_p(PurpleAccount *account, PurpleStatus *status)
 	FacebookAccount *fba = account->gc->proto_data;
 
 	/* if "away" set idle */
-	if (fba && purple_status_type_get_primitive(purple_status_get_type(status)) == PURPLE_STATUS_AWAY)
+	if (purple_status_type_get_primitive(purple_status_get_type(status)) == PURPLE_STATUS_AWAY)
 	{
-		fba->is_idle = TRUE;	
+		if (fba)
+		{
+			fba->is_idle = TRUE;
+		}
+		return;
 	}
 
 	/* first check that we actually want to set this through Pidgin */
