@@ -21,10 +21,13 @@
 #ifndef LIBFACEBOOK_H
 #define LIBFACEBOOK_H
 
-#define FACEBOOK_PLUGIN_VERSION "1.64"
+#define FACEBOOK_PLUGIN_VERSION "1.67"
 #define FACEBOOK_PLUGIN_ID "prpl-bigbrownchunx-facebookim"
 #define FACEBOOK_CAPTCHA_SITE "6LezHAAAAAAAADqVjseQ3ctG3ocfQs2Elo1FTa_a"
 
+/* Maximum number of simultaneous connections to a server */
+#define FB_MAX_CONNECTIONS 32
+
 #include <glib.h>
 
 #include <errno.h>
@@ -77,12 +80,14 @@
 typedef struct _FacebookAccount FacebookAccount;
 typedef struct _FacebookBuddy FacebookBuddy;
 
-typedef void (*FacebookProxyCallbackFunc)(FacebookAccount *fba, gchar *data, gsize data_len, gpointer user_data);
+typedef void (*FacebookProxyCallbackFunc)(FacebookAccount *fba, const gchar *data, gsize data_len, gpointer user_data);
+typedef void (*FacebookFunc)(FacebookAccount *fba);
 
 struct _FacebookAccount {
 	PurpleAccount *account;
 	PurpleConnection *pc;
 	GSList *conns; /**< A list of all active FacebookConnections */
+	GQueue *waiting_conns; /**< A list of all FacebookConnections waiting to process */
 	GSList *dns_queries;
 	GHashTable *cookie_table;
 	gchar *post_form_id;
@@ -122,4 +127,10 @@ struct _FacebookBuddy {
 	gchar *thumb_url;
 };
 
+
+gboolean fb_do_http_login(FacebookAccount *fba, 
+		FacebookProxyCallbackFunc callback_func, gpointer user_data);
+
+/* These functions here are public for libfbxmpp.c */
+
 #endif /* LIBFACEBOOK_H */
